@@ -18,11 +18,14 @@ from pysteps.exceptions import MissingOptionalDependency
 
 try:
     import cv2
+
     cv2_imported = True
 except ImportError:
     cv2_imported = False
 import scipy.spatial
 import time
+import warnings
+
 
 def dense_lucaskanade(R, **kwargs):
     """
@@ -55,6 +58,13 @@ def dense_lucaskanade(R, **kwargs):
 
     Other Parameters
     ----------------
+    dense : bool, optional
+        If True (the default), it returns the three-dimensional array (2,m,n)
+        containing the dense x- and y-components of the motion field. If false,
+        it returns the sparse motion vectors as 1D arrays x, y, u, v, where
+        x, y define the vector locations, u, v define the x and y direction
+        components of the vectors.
+
     buffer_mask : int, optional
         A mask buffer width in pixels. This extends the input mask (if any)
         to help avoiding the erroneous interpretation of velocities near the
@@ -92,11 +102,21 @@ def dense_lucaskanade(R, **kwargs):
         It represents the 0-based maximal pyramid level number, by default this
         is set to 3.
 
-    nr_IQR_outlier : int, optional
-        Maximum acceptable deviation from the median velocity value in terms of
-        number of inter quantile ranges (IQR). Any velocity that is larger than
-        this value is flagged as outlier and excldued from the interpolation.
+    nr_std_outlier : int, optional
+        Maximum acceptable deviation from the mean/median in terms of
+        number of standard deviations. Any anomaly larger than
+        this value is flagged as outlier and excluded from the interpolation.
         By default this is set to 3.
+        
+    multivariate_outlier : bool, optional
+        If true (the default), the outlier detection is computed in terms of
+        the Mahalanobis distance. If false, the outlier detection is simply 
+        computed in terms of velocity.
+        
+    k_outlier : int, optional
+        The number of nearest neighbours used to localize the outlier detection.
+        If set equal to 0, it employs all the data points.
+        The default is 30.
 
     size_opening : int, optional
         The size of the structuring element kernel in pixels. This is used to
@@ -124,8 +144,8 @@ def dense_lucaskanade(R, **kwargs):
         "gaussian".
 
     k : int, optional
-        The number of nearest neighbors used for fast interpolation, by default
-        this is set to 20. If set equal to zero, it employs all the neighbors.
+        The number of nearest neighbours used for fast interpolation, by default
+        this is set to 20. If set equal to zero, it employs all the neighbours.
 
     epsilon : float, optional
         The adjustable constant used in the gaussian and inverse radial basis
@@ -151,10 +171,14 @@ def dense_lucaskanade(R, **kwargs):
     Returns
     -------
     out : ndarray_
-        Three-dimensional array (2,m,n) containing the dense x- and y-components
-        of the motion field.
+        If dense=True (the default), it returns the three-dimensional array (2,m,n)
+        containing the dense x- and y-components of the motion field in units of
+        pixels / timestep as given by the input array R.
+        If dense=False, it returns a tuple containing the one-dimensional arrays
+        x, y, u, v, where x, y define the vector locations, u, v define the x
+        and y direction components of the vectors.
         Return an empty array when no motion vectors are found.
-        
+
     References
     ----------
     
@@ -169,34 +193,56 @@ def dense_lucaskanade(R, **kwargs):
     """
 
     if len(R.shape) != 3:
-        raise ValueError("R has %i dimensions, but a three-dimensional array is expected" % len(R.shape))
+        raise ValueError(
+            "R has %i dimensions, but a three-dimensional array is expected"
+            % len(R.shape)
+        )
 
     if R.shape[0] < 2:
-        raise ValueError("R has %i frame, but at least two frames are expected" % R.shape[0])
+        raise ValueError(
+            "R has %i frame, but at least two frames are expected" % R.shape[0]
+        )
+
+    R = R.copy()
 
     # defaults
+    dense = kwargs.get("dense", True)
     max_corners_ST = kwargs.get("max_corners_ST", 500)
     quality_level_ST = kwargs.get("quality_level_ST", 0.1)
     min_distance_ST = kwargs.get("min_distance_ST", 3)
     block_size_ST = kwargs.get("block_size_ST", 15)
-    winsize_LK = kwargs.get("winsize_LK5", (50, 50))
+    winsize_LK = kwargs.get("winsize_LK", (50, 50))
     nr_levels_LK = kwargs.get("nr_levels_LK", 3)
-    nr_IQR_outlier = kwargs.get("nr_IQR_outlier", 3)
+    nr_std_outlier = kwargs.get("nr_std_outlier", 3)
+    nr_IQR_outlier = kwargs.get("nr_IQR_outlier", None)
+    if nr_IQR_outlier is not None:
+        nr_std_outlier = nr_IQR_outlier
+        warnings.warn(
+            "the 'nr_IQR_outlier' argument will be deprecated in the next release; "
+            + "use 'nr_std_outlier' instead.",
+            category=FutureWarning,
+        )
+    multivariate_outlier = kwargs.get("multivariate_outlier", True)
+    k_outlier = kwargs.get("k_outlier", 30)
     size_opening = kwargs.get("size_opening", 3)
     decl_grid = kwargs.get("decl_grid", 20)
     min_nr_samples = kwargs.get("min_nr_samples", 2)
     rbfunction = kwargs.get("rbfunction", "inverse")
-    k = kwargs.get("k", 20)
+    k = kwargs.get("k", 50)
     epsilon = kwargs.get("epsilon", None)
     nchunks = kwargs.get("nchunks", 5)
     extra_vectors = kwargs.get("extra_vectors", None)
     if extra_vectors is not None:
         if len(extra_vectors.shape) != 2:
-            raise ValueError("extra_vectors has %i dimensions, but 2 dimensions are expected"
-                            % len(extra_vectors.shape))
+            raise ValueError(
+                "extra_vectors has %i dimensions, but 2 dimensions are expected"
+                % len(extra_vectors.shape)
+            )
         if extra_vectors.shape[1] != 4:
-            raise ValueError("extra_vectors has %i columns, but 4 columns are expected"
-                               % extra_vectors.shape[1])
+            raise ValueError(
+                "extra_vectors has %i columns, but 4 columns are expected"
+                % extra_vectors.shape[1]
+            )
     verbose = kwargs.get("verbose", True)
     buffer_mask = kwargs.get("buffer_mask", 0)
 
@@ -229,8 +275,8 @@ def dense_lucaskanade(R, **kwargs):
             continue
 
         # scale between 0 and 255
-        prvs = (prvs - prvs.min())/(prvs.max() - prvs.min())*255
-        next = (next - next.min())/(next.max() - next.min())*255
+        prvs = (prvs - prvs.min()) / (prvs.max() - prvs.min()) * 255
+        next = (next - next.min()) / (next.max() - next.min()) * 255
 
         # convert to 8-bit
         prvs = np.ndarray.astype(prvs, "uint8")
@@ -240,9 +286,9 @@ def dense_lucaskanade(R, **kwargs):
         # buffer the quality mask to ensure that no vectors are computed nearby
         # the edges of the radar mask
         if buffer_mask > 0:
-            mask_ = cv2.morphologyEx(mask_, cv2.MORPH_DILATE,
-                                     np.ones((int(buffer_mask), int(buffer_mask)),
-                                     np.uint8))
+            mask_ = cv2.dilate(
+                mask_, np.ones((int(buffer_mask), int(buffer_mask)), np.uint8), 1
+            )
 
         # remove small noise with a morphological operator (opening)
         if size_opening > 0:
@@ -251,42 +297,41 @@ def dense_lucaskanade(R, **kwargs):
 
         # Shi-Tomasi good features to track
         # TODO: implement different feature detection algorithms (e.g. Harris)
-        mask_ = (-1*mask_ + 1).astype('uint8')
-        p0 = _ShiTomasi_features_to_track(prvs, max_corners_ST, quality_level_ST,
-                                          min_distance_ST, block_size_ST, mask_)
+        mask_ = (-1 * mask_ + 1).astype("uint8")
+        p0 = _ShiTomasi_features_to_track(
+            prvs,
+            max_corners_ST,
+            quality_level_ST,
+            min_distance_ST,
+            block_size_ST,
+            mask_,
+        )
 
         # skip loop if no features to track
         if p0 is None:
             continue
 
         # get sparse u, v vectors with Lucas-Kanade tracking
-        x0, y0, u, v = _LucasKanade_features_tracking(prvs, next, p0, winsize_LK,
-                                                     nr_levels_LK)
+        x0, y0, u, v = _LucasKanade_features_tracking(
+            prvs, next, p0, winsize_LK, nr_levels_LK
+        )
         # skip loop if no vectors
         if x0 is None:
             continue
 
-        # exclude outlier vectors
-        vel = np.sqrt(u**2 + v**2) # [px/timesteps]
-        q1, q2 = np.percentile(vel, [25, 75])
-        min_speed_thr = np.max((0, q1 - nr_IQR_outlier*(q2 - q1)))
-        max_speed_thr = q2 + nr_IQR_outlier*(q2 - q1)
-        keep = np.logical_and(vel < max_speed_thr, vel > min_speed_thr)
-
-        u = u[keep][:, None]
-        v = v[keep][:, None]
-        y0 = y0[keep][:, None]
-        x0 = x0[keep][:, None]
-
-        # stack vectors within time window
-        x0Stack.append(x0)
-        y0Stack.append(y0)
-        uStack.append(u)
-        vStack.append(v)
+        # stack vectors within time window as column vectors
+        x0Stack.append(x0.flatten()[:, None])
+        y0Stack.append(y0.flatten()[:, None])
+        uStack.append(u.flatten()[:, None])
+        vStack.append(v.flatten()[:, None])
 
     # return zero motion field is no sparse vectors are found
     if len(x0Stack) == 0:
-        return np.zeros((2, domain_size[0], domain_size[1]))
+        if dense:
+            return np.zeros((2, domain_size[0], domain_size[1]))
+        else:
+            rzero = np.array([0])
+            return rzero, rzero, rzero, rzero
 
     # convert lists of arrays into single arrays
     x = np.vstack(x0Stack)
@@ -294,8 +339,17 @@ def dense_lucaskanade(R, **kwargs):
     u = np.vstack(uStack)
     v = np.vstack(vStack)
 
+    # exclude outlier vectors
+    x, y, u, v = _outlier_removal(
+        x, y, u, v, nr_std_outlier, multivariate_outlier, k_outlier, verbose
+    )
+
     if verbose:
-        print("--- LK found %i sparse vectors ---" % x0.size)
+        print("--- LK found %i sparse vectors ---" % x.size)
+
+    # return sparse vectors if required
+    if not dense:
+        return x, y, u, v
 
     # decluster sparse motion vectors
     if decl_grid > 1:
@@ -308,17 +362,25 @@ def dense_lucaskanade(R, **kwargs):
         u = np.concatenate((u, extra_vectors[:, 2]))
         v = np.concatenate((v, extra_vectors[:, 3]))
 
-    # return zero motion field is no sparse vectors are left for interpolation
+    # return zero motion field if no sparse vectors are left for interpolation
     if x.size == 0:
         return np.zeros((2, domain_size[0], domain_size[1]))
 
     if verbose:
-        print("--- %i sparse vectors left for interpolation ---" % x.size)
+        print("--- %i sparse vectors left after declustering ---" % x.size)
 
     # kernel interpolation
-    _, _, UV = _interpolate_sparse_vectors(x, y, u, v, domain_size, rbfunction=rbfunction,
-                                           k=k, epsilon=epsilon, nchunks=nchunks)
-
+    _, _, UV = _interpolate_sparse_vectors(
+        x,
+        y,
+        u,
+        v,
+        domain_size,
+        rbfunction=rbfunction,
+        k=k,
+        epsilon=epsilon,
+        nchunks=nchunks,
+    )
 
     if verbose:
         print("--- %.2f seconds ---" % (time.time() - t0))
@@ -326,8 +388,9 @@ def dense_lucaskanade(R, **kwargs):
     return UV
 
 
-def _ShiTomasi_features_to_track(R, max_corners_ST, quality_level_ST,
-                                 min_distance_ST, block_size_ST, mask):
+def _ShiTomasi_features_to_track(
+    R, max_corners_ST, quality_level_ST, min_distance_ST, block_size_ST, mask
+):
     """Call the Shi-Tomasi corner detection algorithm.
 
     Parameters
@@ -364,7 +427,8 @@ def _ShiTomasi_features_to_track(R, max_corners_ST, quality_level_ST,
     if not cv2_imported:
         raise MissingOptionalDependency(
             "opencv package is required for the Lucas-Kanade "
-            "optical flow method but it is not installed")
+            "optical flow method but it is not installed"
+        )
 
     if len(R.shape) != 2:
         raise ValueError("R must be a two-dimensional array")
@@ -372,8 +436,12 @@ def _ShiTomasi_features_to_track(R, max_corners_ST, quality_level_ST,
         raise ValueError("R must be passed as 8-bit image")
 
     # ShiTomasi corner detection parameters
-    ShiTomasi_params = dict(maxCorners=max_corners_ST, qualityLevel=quality_level_ST,
-                            minDistance=min_distance_ST, blockSize=block_size_ST)
+    ShiTomasi_params = dict(
+        maxCorners=max_corners_ST,
+        qualityLevel=quality_level_ST,
+        minDistance=min_distance_ST,
+        blockSize=block_size_ST,
+    )
 
     # detect corners
     p0 = cv2.goodFeaturesToTrack(R, mask=mask, **ShiTomasi_params)
@@ -415,11 +483,15 @@ def _LucasKanade_features_tracking(prvs, next, p0, winsize_LK, nr_levels_LK):
     if not cv2_imported:
         raise MissingOptionalDependency(
             "opencv package is required for the Lucas-Kanade method "
-            "optical flow method but it is not installed")
+            "optical flow method but it is not installed"
+        )
 
     # LK parameters
-    lk_params = dict(winSize=winsize_LK, maxLevel=nr_levels_LK,
-                     criteria=(cv2.TERM_CRITERIA_EPS|cv2.TERM_CRITERIA_COUNT, 10, 0))
+    lk_params = dict(
+        winSize=winsize_LK,
+        maxLevel=nr_levels_LK,
+        criteria=(cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 10, 0),
+    )
 
     # Lucas-Kande
     p1, st, err = cv2.calcOpticalFlowPyrLK(prvs, next, p0, None, **lk_params)
@@ -463,10 +535,11 @@ def _clean_image(R, n=3, thr=0):
     if not cv2_imported:
         raise MissingOptionalDependency(
             "opencv package is required for the Lucas-Kanade method "
-            "optical flow method but it is not installed")
+            "optical flow method but it is not installed"
+        )
 
     # convert to binary image (rain/no rain)
-    field_bin = np.ndarray.astype(R > thr,"uint8")
+    field_bin = np.ndarray.astype(R > thr, "uint8")
 
     # build a structuring element of size (nx)
     kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (n, n))
@@ -483,6 +556,110 @@ def _clean_image(R, n=3, thr=0):
     return R
 
 
+def _outlier_removal(x, y, u, v, thr, multivariate=True, k=30, verbose=False):
+
+    """Outlier removal.
+    
+    Parameters
+    ----------
+    x : array_like
+        X-coordinates of the origins of the velocity vectors.
+    y : array_like
+        Y-coordinates of the origins of the velocity vectors.
+    u : array_like
+        X-components of the velocities.
+    v : array_like
+        Y-components of the velocities.
+    thr : float
+        Threshold for outlier detection defined as measure of deviation from
+        the mean/median in terms of standard deviations.
+    multivariate : bool, optional
+        If true (the default), the outlier detection is computed in terms of
+        the Mahalanobis distance. If false, the outlier detection is simply 
+        computed in terms of velocity.
+    k : int, optinal
+        The number of nearest neighbours used to localize the outlier detection.
+        If set equal to 0, it employs all the data points.
+        The default is 30.
+
+    Returns
+    -------
+        A four-element tuple (x,y,u,v) containing the x- and y-coordinates and
+        velocity components of the motion vectors.
+    """
+
+    if multivariate:
+        data = np.concatenate((u, v), axis=1)
+
+    # globally
+    if k <= 0:
+
+        if not multivariate:
+
+            # in terms of velocity
+
+            vel = np.sqrt(u ** 2 + v ** 2)  # [px/timesteps]
+            q1, q2, q3 = np.percentile(vel, [16, 50, 84])
+            min_speed_thr = np.max((0, q2 - thr * (q3 - q1) / 2))
+            max_speed_thr = q2 + thr * (q3 - q1) / 2
+            keep = np.logical_and(vel < max_speed_thr, vel >= min_speed_thr)
+
+        else:
+
+            # mahalanobis distance
+
+            data = data - np.mean(data, axis=0)
+            V = np.cov(data.T)
+            VI = np.linalg.inv(V)
+            MD = np.sqrt(np.dot(np.dot(data, VI), data.T).diagonal())
+            keep = MD < thr
+
+    # locally
+    else:
+
+        points = np.concatenate((x, y), axis=1)
+        tree = scipy.spatial.cKDTree(points)
+        _, inds = tree.query(points, k=np.min((k + 1, points.shape[0])))
+        keep = []
+        for i in range(inds.shape[0]):
+
+            if not multivariate:
+
+                # in terms of velocity
+
+                thisvel = np.sqrt(u[i] ** 2 + v[i] ** 2)  # [px/timesteps]
+                neighboursvel = np.sqrt(u[inds[i, 1:]] ** 2 + v[inds[i, 1:]] ** 2)
+                q1, q2, q3 = np.percentile(neighboursvel, [16, 50, 84])
+                min_speed_thr = np.max((0, q2 - thr * (q3 - q1) / 2))
+                max_speed_thr = q2 + thr * (q3 - q1) / 2
+                keep.append(thisvel < max_speed_thr and thisvel > min_speed_thr)
+
+            else:
+
+                # mahalanobis distance
+
+                thisdata = data[i, :]
+                neighbours = data[inds[i, 1:], :].copy()
+                thisdata = thisdata - np.mean(neighbours, axis=0)
+                neighbours = neighbours - np.mean(neighbours, axis=0)
+                V = np.cov(neighbours.T)
+                VI = np.linalg.inv(V)
+                MD = np.sqrt(np.dot(np.dot(thisdata, VI), thisdata.T))
+                keep.append(MD < thr)
+
+        keep = np.array(keep)
+
+    if verbose:
+        print("--- %i outliers removed ---" % np.sum(~keep))
+
+    x = x[keep]
+    y = y[keep]
+    u = u[keep]
+    v = v[keep]
+
+    return x, y, u, v
+
+
 def _declustering(x, y, u, v, decl_grid, min_nr_samples):
     """Filter out outliers in a sparse motion field and get more representative
     data points. The method assigns data points to a (RxR) declustering grid
@@ -512,32 +689,37 @@ def _declustering(x, y, u, v, decl_grid, min_nr_samples):
     """
 
     # make sure these are all numpy vertical arrays
-    x = np.atleast_1d(np.array(x).squeeze())[:, None]
-    y = np.atleast_1d(np.array(y).squeeze())[:, None]
-    u = np.atleast_1d(np.array(u).squeeze())[:, None]
-    v = np.atleast_1d(np.array(v).squeeze())[:, None]
+    x = np.array(x).flatten()[:, None]
+    y = np.array(y).flatten()[:, None]
+    u = np.array(u).flatten()[:, None]
+    v = np.array(v).flatten()[:, None]
 
     # return empty arrays if the number of sparse vectors is < min_nr_samples
     if x.size < min_nr_samples:
         return np.array([]), np.array([]), np.array([]), np.array([])
 
     # discretize coordinates into declustering grid
-    xT = x/float(decl_grid)
-    yT = y/float(decl_grid)
+    xT = x / float(decl_grid)
+    yT = y / float(decl_grid)
 
     # round coordinates to low integer
     xT = np.floor(xT)
     yT = np.floor(yT)
 
     # keep only unique combinations of coordinates
-    xy = np.concatenate((xT,yT), axis=1)
-    xyb = np.ascontiguousarray(xy).view(np.dtype((np.void, xy.dtype.itemsize*xy.shape[1])))
-    _,idx = np.unique(xyb, return_index=True)
+    xy = np.concatenate((xT, yT), axis=1)
+    xyb = np.ascontiguousarray(xy).view(
+        np.dtype((np.void, xy.dtype.itemsize * xy.shape[1]))
+    )
+    _, idx = np.unique(xyb, return_index=True)
     uxy = xy[idx]
 
     # now loop through these unique values and average vectors which belong to
     # the same declustering grid cell
-    xN = []; yN = []; uN = []; vN = []
+    xN = []
+    yN = []
+    uN = []
+    vN = []
     for i in range(uxy.shape[0]):
         idx = np.logical_and(xT == uxy[i, 0], yT == uxy[i, 1])
         npoints = np.sum(idx)
@@ -556,8 +738,9 @@ def _declustering(x, y, u, v, decl_grid, min_nr_samples):
     return x, y, u, v
 
 
-def _interpolate_sparse_vectors(x, y, u, v, domain_size, rbfunction="inverse",
-                               k=20, epsilon=None, nchunks=5):
+def _interpolate_sparse_vectors(
+    x, y, u, v, domain_size, rbfunction="inverse", k=20, epsilon=None, nchunks=5
+):
 
     """Interpolation of sparse motion vectors to produce a dense field of motion
     vectors.
@@ -579,7 +762,7 @@ def _interpolate_sparse_vectors(x, y, u, v, domain_size, rbfunction="inverse",
         default : inverse
         available : nearest, inverse, gaussian
     k : int or "all"
-        the number of nearest neighbors used to speed-up the interpolation
+        the number of nearest neighbours used to speed-up the interpolation
         If set equal to "all", it employs all the sparse vectors
         default : 20
     epsilon : float
@@ -603,14 +786,14 @@ def _interpolate_sparse_vectors(x, y, u, v, domain_size, rbfunction="inverse",
     """
 
     # make sure these are vertical arrays
-    x = np.atleast_1d(np.array(x).squeeze())[:, None]
-    y = np.atleast_1d(np.array(y).squeeze())[:, None]
-    u = np.atleast_1d(np.array(u).squeeze())[:, None]
-    v = np.atleast_1d(np.array(v).squeeze())[:, None]
-    points  = np.concatenate((x, y), axis=1)
+    x = np.array(x).flatten()[:, None]
+    y = np.array(y).flatten()[:, None]
+    u = np.array(u).flatten()[:, None]
+    v = np.array(v).flatten()[:, None]
+    points = np.concatenate((x, y), axis=1)
     npoints = points.shape[0]
 
-    if len(domain_size)==1:
+    if len(domain_size) == 1:
         domain_size = (domain_size, domain_size)
 
     # generate the grid
@@ -635,30 +818,34 @@ def _interpolate_sparse_vectors(x, y, u, v, domain_size, rbfunction="inverse",
         subgrids = [grid]
 
     # loop subgrids
-    i0=0
-    for i,subgrid in enumerate(subgrids):
+    i0 = 0
+    for i, subgrid in enumerate(subgrids):
 
         idelta = subgrid.shape[0]
 
         if rbfunction.lower() == "nearest":
-            # find indices of the nearest neighbors
+            # find indices of the nearest neighbours
             _, inds = tree.query(subgrid, k=1)
 
-            U[i0:(i0 + idelta)] = u.ravel()[inds]
-            V[i0:(i0 + idelta)] = v.ravel()[inds]
+            U[i0 : (i0 + idelta)] = u.ravel()[inds]
+            V[i0 : (i0 + idelta)] = v.ravel()[inds]
 
         else:
             if k <= 0:
-                d = scipy.spatial.distance.cdist(points, subgrid, "euclidean").transpose()
-                inds = np.arange(u.size)[None, :]*np.ones((subgrid.shape[0],u.size)).astype(int)
+                d = scipy.spatial.distance.cdist(
+                    points, subgrid, "euclidean"
+                ).transpose()
+                inds = np.arange(u.size)[None, :] * np.ones(
+                    (subgrid.shape[0], u.size)
+                ).astype(int)
 
             else:
-                # find indices of the k-nearest neighbors
+                # find indices of the k-nearest neighbours
                 d, inds = tree.query(subgrid, k=k)
 
             if inds.ndim == 1:
                 inds = inds[:, None]
-                d    = d[:, None]
+                d = d[:, None]
 
             # the bandwidth
             if epsilon is None:
@@ -669,17 +856,21 @@ def _interpolate_sparse_vectors(x, y, u, v, domain_size, rbfunction="inverse",
 
             # the interpolation weights
             if rbfunction.lower() == "inverse":
-                w = 1.0/np.sqrt((d/epsilon)**2 + 1)
+                w = 1.0 / np.sqrt((d / epsilon) ** 2 + 1)
             elif rbfunction.lower() == "gaussian":
-                w = np.exp(-0.5*(d/epsilon)**2)
+                w = np.exp(-0.5 * (d / epsilon) ** 2)
             else:
                 raise ValueError("unknown radial fucntion %s" % rbfunction)
 
             if not np.all(np.sum(w, axis=1)):
-                w[np.sum(w, axis=1) == 0, :] = 1.
+                w[np.sum(w, axis=1) == 0, :] = 1.0
 
-            U[i0:(i0 + idelta)] = np.sum(w*u.ravel()[inds], axis=1)/np.sum(w, axis=1)
-            V[i0:(i0 + idelta)] = np.sum(w*v.ravel()[inds], axis=1)/np.sum(w, axis=1)
+            U[i0 : (i0 + idelta)] = np.sum(w * u.ravel()[inds], axis=1) / np.sum(
+                w, axis=1
+            )
+            V[i0 : (i0 + idelta)] = np.sum(w * v.ravel()[inds], axis=1) / np.sum(
+                w, axis=1
+            )
 
         i0 += idelta
 
